@@ -1,17 +1,22 @@
 import calendar
+import collections
+import copy
+import csv
 import dataclasses
 import datetime
+import io
 import logging
-import os
 import pathlib
 import textwrap
 
 import durations
+import holidays
 import inflect
 import jinja2
 import pkg_resources
 import timeago
 
+from . import config as configmod
 from . import model
 
 _logger = logging.getLogger(__name__)
@@ -21,12 +26,30 @@ templates_dir = pathlib.Path(pkg_resources.resource_filename(package, "templates
 loader = jinja2.FileSystemLoader(searchpath=templates_dir)
 env = jinja2.Environment(loader=loader, keep_trailing_newline=True)
 
+sec_per_hour = datetime.timedelta(hours=1).total_seconds()
 now = datetime.datetime.now()
 local_now = now.astimezone()
 local_tz = local_now.tzinfo
 local_tzname = local_tz.tzname(local_now)
 delta_net30 = datetime.timedelta(days=30)
-hourly_rate = float(os.environ.get("HOURLY_RATE", 0.0))
+hourly_rate = float(configmod.config["HOURLY_RATE"]) or 0.0
+
+
+def get_net30(start_date: datetime.datetime, net30=delta_net30) -> datetime.timedelta:
+    """
+    Given some date start_date, return the timedelta till the non-holiday monday
+    after (start_date + 30 days).
+    """
+    us_holidays = holidays.US(years=start_date.year)
+
+    def is_holiday_or_weekend(date):
+        return date.weekday() >= 5 or date in us_holidays
+
+    next_weekday = start_date + net30
+    while is_holiday_or_weekend(next_weekday):
+        next_weekday += datetime.timedelta(days=1)
+
+    return next_weekday - start_date
 
 
 @dataclasses.dataclass
@@ -36,7 +59,7 @@ class Thingy:
     data: dict
 
 
-def view_hours_per_task(timesheet: model.Timesheet):
+def view_hours_per_task(timesheet: model.Timesheet) -> str:
     names = {}
     invoices = set()
     for entry in timesheet.days:
@@ -62,7 +85,7 @@ def view_hours_per_task(timesheet: model.Timesheet):
         label_total = f"invoices {', '.join([str(x) for x in invoices])} total"
 
     template = env.get_template("view_hours_worked_per_task.j2")
-    out = template.render(
+    inv_str = template.render(
         data={
             "invoices": invoices,
             "stuff": stuff,
@@ -70,10 +93,10 @@ def view_hours_per_task(timesheet: model.Timesheet):
             "label_total": label_total,
         }
     )
-    return out
+    return inv_str
 
 
-def view_hours_worked_per_day(timesheet: model.Timesheet):
+def view_hours_worked_per_day(timesheet: model.Timesheet) -> str:
     stuff = []
 
     for entry in timesheet.days:
@@ -110,11 +133,10 @@ def view_hours_worked_per_day(timesheet: model.Timesheet):
 
     template = env.get_template("view_hours_worked_per_day.j2")
     stuff = sorted(stuff, key=lambda i: i["date"], reverse=True)
-    out = template.render(data=stuff)
-    return out
+    return template.render(data=stuff)
 
 
-def view_hours_worked_per_day_summary(timesheet: model.Timesheet):
+def view_hours_worked_per_day_summary(timesheet: model.Timesheet) -> str:
     daily_entries = []
 
     total_time_worked = datetime.timedelta(seconds=0)
@@ -126,7 +148,7 @@ def view_hours_worked_per_day_summary(timesheet: model.Timesheet):
 
         total_time_worked += datetime.timedelta(seconds=seconds)
 
-        earned = hourly_rate * total_time_worked.total_seconds() / 60 / 60
+        earned = hourly_rate * total_time_worked.total_seconds() / sec_per_hour
         earned = "${:,.2f}".format(earned)
         earned = "{:>10}".format(earned)
 
@@ -142,23 +164,23 @@ def view_hours_worked_per_day_summary(timesheet: model.Timesheet):
     template = env.get_template("view_hours_worked_per_day_summary.j2")
     daily_entries = sorted(daily_entries, key=lambda i: i["date"], reverse=True)
 
-    x = total_time_worked.total_seconds() / 60 / 60
+    x = total_time_worked.total_seconds() / sec_per_hour
     total_time_worked_friendly = (
         "{:d}h".format(int(x)) if int(x) == x else "{0:.2f}h".format(x)
     )
 
-    out = template.render(
+    view_str = template.render(
         data={
             "summary": {"total_time_worked_friendly": total_time_worked_friendly},
             "entries": daily_entries,
         }
     )
-    return out
+    return view_str
 
 
 def timedelta_to_short_string(td: datetime.timedelta) -> str:
     seconds = td.total_seconds()
-    hours, remainder = divmod(seconds, 3600)
+    hours, remainder = divmod(seconds, sec_per_hour)
     minutes, seconds = divmod(remainder, 60)
     result = ""
     if hours > 0:
@@ -170,7 +192,7 @@ def timedelta_to_short_string(td: datetime.timedelta) -> str:
     return result.strip()
 
 
-def view_csv(timesheet: model.Timesheet):
+def generate_csv_data(timesheet: model.Timesheet) -> list[dict]:
     stuff = []
     for entry in timesheet.days:
         for task in entry.tasks.__root__:
@@ -183,15 +205,13 @@ def view_csv(timesheet: model.Timesheet):
             x1 = {
                 "task": task.task,
                 "date": entry.date,
-                "worked_time": duration.to_seconds() / 60 / 60,
+                "worked_time": duration.to_seconds() / sec_per_hour,
                 "worked_time_friendly": task.task_time,
                 "invoice": entry.invoice,
                 "minutia": minutia,
             }
             stuff.append(x1)
 
-    template = env.get_template("view_csv.j2")
-
     tasks = sorted(stuff, key=lambda i: i["date"], reverse=True)
 
     invoice = None
@@ -203,13 +223,64 @@ def view_csv(timesheet: model.Timesheet):
         total_per_invoice += duration.to_seconds()
         delta = datetime.timedelta(seconds=total_per_invoice)
         task["worked_time_cumulative"] = timedelta_to_short_string(delta)
-        task["worked_time_cumulative_frac"] = total_per_invoice / 60 / 60
+        task["worked_time_cumulative_frac"] = total_per_invoice / sec_per_hour
 
-    out = template.render(tasks=tasks)
-    return out
+    return tasks
 
 
-def view_invoices(timesheet: model.Timesheet):
+def view_csv_stringio(tasks: list[dict]) -> io.StringIO:
+    tasks = copy.deepcopy(tasks)
+    headers = collections.OrderedDict(
+        {
+            "date": "date",
+            "invoice": "invoice",
+            "inv total": "worked_time_cumulative",
+            "duration": "worked_time_friendly",
+            "task": "task_details_pretty",
+        }
+    )
+
+    data = []
+    for task in tasks:
+        x1 = task["date"].strftime("%a %m-%d")
+        x2 = task["worked_time_friendly"]
+        x3 = task["worked_time"]
+        x4 = task["worked_time_cumulative"]
+        x5 = task["worked_time_cumulative_frac"]
+
+        task["date"] = x1
+        task["worked_time_cumulative"] = f"{x4} ({x5:.2f})"
+        task["worked_time_friendly"] = f"{x2} ({x3:.2f})"
+        task["task_details_pretty"] = (
+            f"{task['task']} - {task['minutia']}" if task["minutia"] else task["task"]
+        )
+        dct = {}
+        for header, key in headers.items():
+            dct[header] = task[key]
+
+        data.append(dct)
+
+    my_stringio = io.StringIO()
+    writer = csv.DictWriter(my_stringio, fieldnames=headers.keys())
+    writer.writeheader()
+    for row in data:
+        writer.writerow(row)
+
+    return my_stringio
+
+
+def view_csv_jinja2(timesheet: model.Timesheet) -> str:
+    tasks = generate_csv_data(timesheet)
+    template = env.get_template("view_csv.j2")
+    return template.render(tasks=tasks)
+
+
+def view_google_sheets(timesheet: model.Timesheet) -> str:
+    tasks = generate_csv_data(timesheet)
+    return view_csv_stringio(tasks).getvalue()
+
+
+def view_invoices(timesheet: model.Timesheet) -> str:
     template = env.get_template("view_invoices.j2")
     invoices = timesheet.invoices.__root__
     invoices_by_inv_number = sorted(invoices, key=lambda x: x.number, reverse=False)
@@ -243,10 +314,11 @@ def view_invoices(timesheet: model.Timesheet):
         submitted_on = None
 
         if invoice.submitted_on is not None:
-            due_date = invoice.submitted_on + delta_net30
-            delta = due_date - local_now
+            s = invoice.submitted_on
+            due_date = s + get_net30(s, delta_net30)
+            delta = due_date - local_now + datetime.timedelta(days=1)
             submitted_on = invoice.submitted_on.date()
-            ts = invoice.submitted_on + delta_net30
+            ts = s + get_net30(s, delta_net30)
             days = inflect.engine().plural("day", delta.days)
             date = ts.strftime("%m-%d")
             payout_due_relative = f"in {delta.days} {days} on {date}"
@@ -276,5 +348,4 @@ def view_invoices(timesheet: model.Timesheet):
 
         display_dicts.append(display)
 
-    out = template.render(data=display_dicts)
-    return out
+    return template.render(data=display_dicts)
