@@ -30,7 +30,8 @@ import yaml
 
 from foundationlive import __version__, lib
 
-from . import model
+from . import config as configmod
+from . import googlesheets, model
 from . import writer as writermod
 
 __author__ = "Taylor Monacelli"
@@ -93,7 +94,10 @@ def parse_args(args):
         const=logging.DEBUG,
     )
     parser.add_argument(
-        "--data-path", default="data.json", required=True, help="path to data.json"
+        "--data-path",
+        default=configmod.config["FOUNDATIONLIVE_DATA_PATH"],
+        required=False,
+        help="path to data.yaml",
     )
     parser.add_argument(
         "-i",
@@ -103,6 +107,19 @@ def parse_args(args):
         action="append",
         help="Remove all task entries that don't match invoice these invoice numbers",
     )
+    parser.add_argument(
+        "--google-sheets",
+        required=False,
+        action="store_true",
+        help="update google sheets",
+    )
+    parser.add_argument(
+        "--show-config",
+        required=False,
+        action="store_true",
+        help="display details of how this app is configured",
+    )
+
     return parser.parse_args(args)
 
 
@@ -131,7 +148,14 @@ def main(args):
     args = parse_args(args)
     setup_logging(args.loglevel)
     _logger.debug("Starting crazy calculations...")
+    configmod.init()
+
     records_path = pathlib.Path(args.data_path)
+    if not records_path.exists():
+        msg = f"{records_path} is missing"
+        _logger.critical(msg)
+        sys.exit(-1)
+
     with open(records_path, "r") as stream:
         try:
             external_data = yaml.safe_load(stream)
@@ -160,17 +184,28 @@ def main(args):
             lib.view_hours_worked_per_day_summary,
             timesheet_filtered,
         ),
-        lib.Thingy("view_csv.txt", lib.view_csv, timesheet_filtered),
+        lib.Thingy("view_csv.txt", lib.view_csv_jinja2, timesheet_filtered),
         # want timesheet for view_invoices instead of timesheet_filtered
         lib.Thingy("view_invoices.txt", lib.view_invoices, timesheet),
     ]
 
+    reports_output_dir = pathlib.Path(
+        configmod.config["FOUNDATIONLIVE_TEMPLATES_OUTPUT_DIRECTORY"]
+    ).expanduser()
+
     for thing in outputs:
         out = thing.fn(thing.data)
-        writermod.FileWriter(thing.fname).write(out)
-        writermod.ConsoleWriter().write(out)
+        out_path = reports_output_dir / thing.fname
+        _logger.debug(f"writing {out_path}")
+        writermod.FileWriter(out_path).write(out)
+    #        writermod.ConsoleWriter().write(out)
 
-    _logger.info("Script ends here")
+    if args.google_sheets:
+        out = lib.view_google_sheets(timesheet_filtered)
+        googlesheets.main(out)
+
+    if args.show_config:
+        configmod.show_config()
 
 
 def run():
